# Additional input columns
FrameVariantIndices = "frames"
FrameConnotations = "frame_connots_uint"
SynonymObject = "syn_objs"
SynonymSubject = "syn_subjs"
PosTags = "pos_tags"
Text = "text"

ArgsSep = ','
