"""
paper: Independently Recurrent Neural Network (IndRNN): Building A Longer and Deeper RNN
authors: Shuai Li, Wanqing Li, Chris Cook, Ce Zhu, Yanbo Gao
paper link: https://arxiv.org/pdf/1803.04831.pdf
code: https://github.com/batzner/indrnn
"""
# TODO. copy cell
