class BaseSamplesIO(object):
    """ Represents base experiment utils for input/output for:
        samples -- data that utilized for experiments;
        results -- evaluation of experiments.
    """

    @property
    def Reader(self):
        raise NotImplementedError()

    def create_target(self, data_type):
        """ Path for reading/viewing
        """
        raise NotImplementedError()
