ID = 'id'
LABEL = 'label'
OPINION_INDEX = 'opinion_id'

# Corresponds to fields with attitude ends. (values, STRING)
SOURCE = 'source'
TARGET = 'target'

# Corresponds to fields with attitude ends. (indices, INT)
S_IND = 's_ind'
T_IND = 't_ind'

# Entity parameters
ENTITIES = 'entities'
